# Python imports

# Lib imports

# Application imports



class ImageViewMixin:
    def _zoom_out(self):
        if self.work_pixbuf and self.pixbuf:
            # TODO: Setup scale factor setting to pull from settings...
            stepx = self.work_pixbuf.get_width()  * 0.05
            stepy = self.work_pixbuf.get_height() * 0.05

            w = self.work_pixbuf.get_width()  - stepx
            h = self.work_pixbuf.get_height() - stepy

            self.work_pixbuf = self.pixbuf.scale_simple(w, h, 2)  # 2 = BILINEAR and is best by default
            self.set_from_pixbuf(self.work_pixbuf)

    def _rotate_left(self):
        if self.work_pixbuf and self.pixbuf:
            self.work_pixbuf = self.work_pixbuf.rotate_simple(GdkPixbuf.PixbufRotation.COUNTERCLOCKWISE)
            self.pixbuf      = self.pixbuf.rotate_simple(GdkPixbuf.PixbufRotation.COUNTERCLOCKWISE)
            self.set_from_pixbuf(self.work_pixbuf)

    def _vertical_flip(self):
        if self.work_pixbuf and self.pixbuf:
            self.work_pixbuf = self.work_pixbuf.flip(True)
            self.pixbuf      = self.pixbuf.flip(True)
            self.set_from_pixbuf(self.work_pixbuf)

    def _scale_1_two_1(self):
        self.fit_to_win = False
        if self.work_pixbuf and self.pixbuf:
            self.work_pixbuf = self.pixbuf
            self.set_from_pixbuf(self.work_pixbuf)

    def _fit_to_container(self):
        self.fit_to_win = True
        if self.work_pixbuf and self.pixbuf:
            parent_aloc = self.get_parent().get_parent().get_allocation()
            pw = parent_aloc.width
            ph = parent_aloc.height
            iw = self.pixbuf.get_width()
            ih = self.pixbuf.get_height()
            w  = 0
            h  = 0

            if iw == 0 or ih == 0:
                return

            w = pw;
            h = ((ih * w) / iw + 0.5)

            if h > ph:
                h = ph
                w = (iw * h) / ih + 0.5


            self.work_pixbuf = self.pixbuf.scale_simple(w, h, 2)  # 2 = BILINEAR and is best by default
            self.set_from_pixbuf(self.work_pixbuf)

    def _horizontal_flip(self):
        if self.work_pixbuf and self.pixbuf:
            self.work_pixbuf = self.work_pixbuf.flip(False)
            self.pixbuf      = self.pixbuf.flip(False)
            self.set_from_pixbuf(self.work_pixbuf)

    def _rotate_right(self):
        if self.work_pixbuf and self.pixbuf:
            self.work_pixbuf = self.work_pixbuf.rotate_simple(GdkPixbuf.PixbufRotation.CLOCKWISE)
            self.pixbuf      = self.pixbuf.rotate_simple(GdkPixbuf.PixbufRotation.CLOCKWISE)
            self.set_from_pixbuf(self.work_pixbuf)

    def _zoom_in(self):
        if self.work_pixbuf and self.pixbuf:
            # TODO: Setup scale factor setting to pull from settings...
            stepx = self.work_pixbuf.get_width()  * 0.05
            stepy = self.work_pixbuf.get_height() * 0.05

            w = self.work_pixbuf.get_width()  + stepx
            h = self.work_pixbuf.get_height() + stepy

            self.work_pixbuf = self.pixbuf.scale_simple(w, h, 2)  # 2 = BILINEAR and is best by default
            self.set_from_pixbuf(self.work_pixbuf)

    def _size_allocate(self):
        if self.fit_to_win:
            self._fit_to_container()
